##############################################################################
#
# Enfold Stage Deployment - Remote Deployment of Staged Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: Install.py 3550 2005-12-05 17:08:23Z sidnei $
"""

from cStringIO import StringIO
from Products.Archetypes.Extensions.utils import install_subskin
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal
from Products.CMFCore.DirectoryView import createDirectoryView
from Products.StageDeployment.config import *

_marker = object()

PROPERTIES = {
    DEPLOYMENT_TARGET : ('', 'string'),
    }

CONTROL_PANEL = {
    'Stage Deployment':dict(
    id='Stage Deployment',
    name='Stage Deployment',
    action='string:${portal_url}/prefs_stage_deployment',
    condition='',
    permission=ManagePortal,
    category='Products',
    appId='Stage Deployment',
    imageUrl='stage_deployment.gif'),
    }

def setup_properties(self, out):
    title = 'Stage Deployment (portal-level config)'
    print >> out, 'Setting up %s.' % title
    pprops = getToolByName(self, 'portal_properties', _marker)
    if pprops is _marker:
        raise ValueError, "Could not find 'portal_properties' tool"
    pname = 'stage_deployment'
    dprops = getToolByName(pprops, pname, _marker)
    if dprops is _marker:
        print >> out, 'Stage Deployment settings did not exist, creating.'
        # Doesn't exist yet
        pprops.addPropertySheet(pname, title=title)
        dprops = getToolByName(pprops, pname, _marker)
    for pname, psettings in PROPERTIES.items():
        if dprops.hasProperty(pname):
            print >> out, "Property '%s' already exist, skipping." % pname
            continue
        print >> out, "Creating property '%s'." % pname
        dprops.manage_addProperty(pname, *psettings)
    print >> out, '%s setup successfully.' % title

def setup_site_config(self, out):
    cp = getToolByName(self, 'portal_controlpanel')

    existing = [c.id for c in cp._actions]
    for pname, psettings in CONTROL_PANEL.items():
        if not pname in existing:
            print >> out, "Configlet for '%s' did not exist, creating." % pname
            cp.registerConfiglet(**psettings)
    print >> out, "Site Config panel registered successfully"

def install_actions(self, out):
    at = getToolByName(self, 'portal_actions')
    action_ids = [action.getId() for action in at.listActions()]
    if 'stage_deployment_deploy' not in action_ids:
        at.addAction('stage_deployment_deploy',
                     'Deploy a specific version',
                     'string:$object_url/workspace_deploy_form',
                     'python: object.getPortalTypeName() in ("StagingArea",)',
                     'Manage portal',
                     'document_actions')
        out.write('Added action for action icon.\n')
    else:
        out.write('Action was already installed.\n')
    ai = getToolByName(self, 'portal_actionicons')
    if ai.queryActionIcon('plone', 'stage_deployment_deploy', None) is None:
        ai.addActionIcon('plone', 'stage_deployment_deploy',
                         'stage_deployment_deploy_icon.gif',
                         'Deploy a specific version')
        out.write('Added action icon for action.\n')
    else:
        out.write('Action icon was already installed.\n')

def install(self, out=None):
    if out is None:
        out = StringIO()
    setup_properties(self, out)
    setup_site_config(self, out)
    install_subskin(self, out, GLOBALS, product_skins_dir='skins')
    install_actions(self, out)
    return out.getvalue()
