##############################################################################
#
# Enfold Stage Deployment - Remote Deployment of Staged Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: interface.py 3551 2005-12-05 18:33:32Z sidnei $
"""

from zope.interface import Interface

class IInformationExtractor(Interface):

    def extract():
        """Return a simple data structure containing information
        suitable for deployment about the adapted object.
        """

class IReferenceExtractor(Interface):

    def extract():
        """Return information about relationships where this object
        is the 'source'.

        The structure return is a dictionary where keys are the
        'relationships' and values is a list of target UIDs.
        """

class ILocalRolesExtractor(Interface):

    def extract():
        """Return information about local roles for a given object.

        The structure return is a dictionary where keys are the
        'principal ids' and values is a tuple of roles.
        """

class IStage(Interface):
    """Interface of EnSimpleStaging Stage (needs completion)
    """

class INullDeploymentStage(IStage):
    """Marker interface for disabling remote deployment of a stage
    """

class IDeploymentHandler(Interface):

    def open(addr):
        """Open connection to deployment target
        """

    def close():
        """Close connection to deployment target
        """

    def deploy(info):
        """Deploy information to deployment target
        """

