##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py 732 2005-01-21 19:43:40Z sidnei $
"""

__metaclass__ = type

from sqlobject import *
from formencode import validators
from datetime import datetime
from entransit.common import ConnectionDescriptor

class Meta(SQLObject):
    _connection = ConnectionDescriptor()
    _connection_string = None

    class sqlmeta(object):
        cacheValues = False

    # Columns
    path_hash = IntCol(notNull=1)

    path = StringCol(
        length=255, notNull=1, validator=validators.String(strip_spaces=1))

    object_id = StringCol(
        length=255, notNull=1, validator=validators.String(strip_spaces=1))

    modified = DateTimeCol(notNull=1, default=datetime.utcnow)

    # Indexes
    path_hash_index = DatabaseIndex('path_hash')

    path_index = DatabaseIndex('path')

    object_id_index = DatabaseIndex('object_id')

    modified_index = DatabaseIndex('modified')

class CMFMeta(SQLObject):

    _connection = ConnectionDescriptor()
    _connection_string = None
    class sqlmeta(object):
        cacheValues = False

    # Columns
    path_hash = IntCol(notNull=1)

    path = StringCol(
        length=4096, notNull=1, validator=validators.String(strip_spaces=1))

    object_id = StringCol(
        length=255, notNull=1, validator=validators.String(strip_spaces=1))

    uid = StringCol(
        length=255, notNull=1, validator=validators.String(strip_spaces=1))

    modified = DateTimeCol(notNull=1, default=datetime.utcnow)

    created = DateTimeCol(notNull=1, default=datetime.utcnow)

    start_date = DateTimeCol(notNull=1, default=datetime.utcnow)

    end_date = DateTimeCol(notNull=1, default=datetime.utcnow)

    effective_date = DateTimeCol(notNull=1, default=datetime.utcnow)

    expires_date = DateTimeCol(notNull=1, default=datetime.utcnow)

    portal_type = StringCol(
        length=255, notNull=0, default='',
        validator=validators.String(strip_spaces=1))

    meta_type = StringCol(
        length=255, notNull=0, default='',
        validator=validators.String(strip_spaces=1))

    review_state = StringCol(
        length=255, notNull=0, default='',
        validator=validators.String(strip_spaces=1))

    title = UnicodeCol(notNull=0, default=u'')

    description = UnicodeCol(notNull=0, default=u'')

    # Indexes
    path_hash_index = DatabaseIndex('path_hash')

    path_index = DatabaseIndex('path')

    object_id_index = DatabaseIndex('object_id')

    uid_index = DatabaseIndex('uid')

    review_state_index = DatabaseIndex('review_state')

    modified_index = DatabaseIndex('modified')

    created_index = DatabaseIndex('created')

    start_date_index = DatabaseIndex('start_date')

    end_date_index = DatabaseIndex('end_date')

    effective_date_index = DatabaseIndex('effective_date')

    expires_date_index = DatabaseIndex('expires_date')

    portal_type_index = DatabaseIndex('portal_type')

    meta_type_index = DatabaseIndex('meta_type')

    review_state_portal_type_index = DatabaseIndex('review_state',
                                                   'portal_type')

class SQLObjectMeta:

    # Correct values will be stitched in by ZConfig
    name = 'meta-storage'
    impl = None

    def add(self, key, info):
        data = info.get('data', {})
        impl = self.impl
        c_keys = impl.sqlmeta.columns.keys()
        path_hash = hash(key)
        res = impl.select(impl.q.path_hash == path_hash)
        meta = {'path_hash': path_hash}
        if 'path' in c_keys:
            meta['path'] = key
        for k in c_keys:
            if data.has_key(k):
                meta[k] = data[k]
        count = res.count()
        if not count:
            __traceback_info__ = (key, self, impl, meta, data)
            ob = impl(**meta)
        elif count == 1:
            ob = iter(res).next()
            # XXX Seems like there's a bug with _lazyUpdate and
            # setting attributes on existing instances. Maybe some
            # caching issue?
            s = ob._lazyUpdate
            ob._lazyUpdate = False
            ob.set(**meta)
            ob._lazyUpdate = s
        else:
            raise ValueError, 'Duplicate values for key %s' % key

    # Alias update to add.
    update = add

    def remove(self, key, info):
        impl = self.impl
        path_hash = hash(key)
        res = impl.select(impl.q.path_hash == path_hash)
        count = res.count()
        if not count:
            # do nothing, no value exists for key.
            pass
        elif count == 1:
            ob = iter(res).next()
            ob.destroySelf()
        else:
            raise ValueError, 'Duplicate values for key %s' % key

__all__ = ('SQLObjectMeta', 'CMFMeta', 'Meta')
