##############################################################################
#
# Enfold Enterprise Deployment - Remote Deployment of Content
# Copyright (C) 2005 Enfold Systems
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: tracker.py 2983 2005-10-03 22:57:15Z sidnei $
"""

__metaclass__ = type

import os
import os.path
import weakref
import logging
from entransit.utils import make_target_path
from entransit.exceptions import OSError
from entransit.logger import TRACE
from entransit.storage.fs import Add, directory_creation_mode

class Recorder(Add):

    target_must_exist = False

    def __init__(self, target, backup_path):
        super(Recorder, self).__init__(None, target, backup_path)
        dest = make_target_path(self.backup_path, self.target)
        dest_folder = os.path.dirname(dest)
        if not os.path.exists(dest_folder):
            os.makedirs(dest_folder, mode=directory_creation_mode())
        self.payload = dest
        self.out = open(dest, 'wb')

    def execute(self):
        if self.executed:
            # Don't execute twice
            return
        self.out.close()
        super(Recorder, self).execute()

    def cleanup(self):
        super(Recorder, self).cleanup()
        self.executed = False

    def record(self, action, key):
        self.out.write('%s\n%s\n' % (action, key))


def handle(self, op, key, info):
    s_info = info['storage']
    for s_name in self.track:
        s_key = s_info.get(s_name, None)
        if s_key is None:
            continue
        self.recorder.record(op, s_key)

class TrackerStorage:

    _recorder = None
    backup_path = None
    target_path = None

    def __init__(self):
        self.name = 'tracker-storage'
        self.track = ('data-storage', 'blob-storage')

    def _get_recorder(self):
        from entransit.command import after_hooks_queue
        if self._recorder is not None:
            # Dereference
            r = self._recorder()
            if r is not None:
                return r
        fname = os.environ['DEPLOYMENT_START']
        path = os.path.join(self.target_path, fname)
        assert not os.path.exists(path), 'That was too fast.'
        self.log('Creating recorder with path %s.' % path)
        r = Recorder(path, self.backup_path)
        self._recorder = weakref.ref(r)
        after_hooks_queue.queue(r)
        return r
    recorder = property(_get_recorder)

    def log(self, msg):
        logger = logging.getLogger('entransit.%s' % self.name)
        logger.log(TRACE, msg)

    def add(self, key, info):
        handle(self, 'add', key, info)

    def update(self, key, info):
        handle(self, 'update', key, info)

    def remove(self, key, info):
        handle(self, 'remove', key, info)
